package it.openutils.spring.rmibernate.client;

import it.openutils.spring.rmibernate.shared.managers.HibernateLazyService;

import org.aopalliance.intercept.MethodInvocation;
import org.springframework.remoting.rmi.RmiProxyFactoryBean;


/**
 * ProxyFactoryBean for automatic injection of remote lazy loading proxies
 * @author mmolaschi
 * @version $Id: HibernateRmiProxyFactoryBean.java 485 2007-10-26 11:06:52Z fcarone $
 */
public class HibernateRmiProxyFactoryBean extends RmiProxyFactoryBean
{

    /**
     *
     */
    private static final long serialVersionUID = 6545259150454309616L;

    /**
     * Store the hibernate lazy service to be used by remote lazy loading
     */
    public static ThreadLocal<HibernateLazyService> hibernateLazyServiceTL = new ThreadLocal<HibernateLazyService>();

    private HibernateLazyService hibernateLazyService;

    /**
     * {@inheritDoc}
     */
    @Override
    public Object invoke(MethodInvocation invocation) throws Throwable
    {
        // store the lazy service for this thread
        if (hibernateLazyService == null)
        {
            // if hibernateLazyService is null this means that this service IS hibernateLazyService
            hibernateLazyServiceTL.set((HibernateLazyService)this.getObject());
        }
        else
        {
            hibernateLazyServiceTL.set(hibernateLazyService);
        }

        // proceed with rmi call
        return super.invoke(invocation);
    }

    /**
     * Sets the hibernateLazyService.
     * @param hibernateLazyService the hibernateLazyService to set
     */
    public void setHibernateLazyService(HibernateLazyService hibernateLazyService)
    {
        this.hibernateLazyService = hibernateLazyService;
    }

}
